/*
    udos: micro disk OS. 
    Copyright (c) 2003, 2004   Katsuhiko Gondow (gondow@cs.titech.ac.jp)
    $Id: boot16.S,v 1.14 2004/06/23 07:34:44 gondow Exp $
*/

/*
    Note: The size of this code must be less than or equal to 446 bytes;
	  64 bytes for BIOS parameter block, and 2 bytes for the magic
	  number (0xAA55) in the first 512 byte sector. (512-64-2=446)
	  To achieve this constraint, we did some optimization like
	  omitting save/restore registers.
	  Nevertheless, I did not use *smart* code reduction (To tell the
	  truth, I do not know how to do it!).
*/
.code16
.text
.org 64
    ljmp $0x07C0, $1f       # set cs==0x7C0 to start this from offset 0
1:	
    movw %cs,    %ax
    movw %ax,    %ds
    movw %ax,    %ss
    movw $0x8000,%sp	    # to avoid to pollute 0x10000 and higher
    	    	    	    # 0x7C00 + 0x8000 = 0xFC00 < 0x10000
                            # ^^^^^^cs ^^^^^^offset
    movw $0x0800, %ax	    # used for reading root dir. and FAT
    movw %ax,    %fs
    movw $0x1000, %ax       # used for reading the kernel 'KERNEL.BIN'
    movw %ax,    %gs

    # inquire memory size
    movb $0x88,  %ah
    int  $0x15
    movw %ax,    MemorySize # in KB excluding the first 640KB

    call read_udos          # read /KERNEL.BIN to 0x10000 and after

    cli                     # interrupts disabled
    lidt idt_selector
    lgdt gdt_selector

    # set %cr0's PE bit to go into protected-mode.
    movl %cr0,   %eax
    orl  $1,     %eax
    movl %eax,   %cr0

    ljmpl $0x08, $0x10000
2:
    # never reached here
    hlt                     # halt until interrupt/trap occurs
    jmp 2b

read_udos:	
#if 0
    pushw %ax
    pushw %bx
    pushw %cx
    pushw %dx
#endif

    /* FirstRootDirSecNum = BPB_RsvdSecCnt + (BPB_NumFATs * BPB_FATSz16);*/
    movb 0x10,   %al     /* 0x10: BPB_NumFATs */
    movb $0,     %ah     /* convert %al -> %ax */
    movw 0x16,   %bx     /* 0x16: BPB_FATSz16 */
    mulw %bx             /* %bx * %ax -> %dx:%ax (%dx is always 0 for FD) */
    movw 0x0E,   %bx	 /* 0x0E: BPB_RsvdSecCnt */
    addw %bx,    %ax     /* %ax = FirstRootDirSecNum */
    movw %ax,    FirstRootDirSecNum
	
    /* RootDirSectors = ((BPB_RootEntCnt * 32) + (BPB_BytePerSec - 1))
			 / BPB_BytsPerSec; */
    movw $32,    %ax
    mulw 0x11            /* 0x11: BPB_RootEntCnt. */
		         /* %ax * 32 -> %dx:%ax (%dx is always 0 for FD) */
    movw 0x0B,   %bx     /* 0x0B: BPB_BytsPerSec (usually 512) */
    addw %bx,    %ax
    decw %ax
    div  %bx             /* %al = RootDirSectors */
    movb $0,     %ah
    movw %ax,    RootDirSectors

    /* Read root directory to %fs:0000*/
    movw FirstRootDirSecNum, %ax
    call LBA2CHS
    movw %fs,    %bx
    movw %bx,    %es
    movw $0,     %bx 	    # destination is %es:%bx
    movb $0x00,  %dl        # drive number. bit 7 must be 0 for floppy. 
    movb $0x02,  %ah        # command:	read diskette sectors
    movb RootDirSectors, %al # the number of sectors to read
    int  $0x13	      	    # BIOS floppy service

    /* Find boot_file in the root directory */
    movw $0,     %bx
1:	
    cmpb $0,     %fs:(%bx)  # is end-of-directory?
    je 2f

    # compare two filenames
    movw $11,    %cx  
    movw $boot_file,  %si
    movw %bx,    %di
    repe cmpsb              # %cx=counter, (%ds:)%si=source, (%es:)%di=dest. 
                            # repe cmpsb is slow, but it makes code short!
    cmpw $0,     %cx
    je 3f

    # try next dirent
    addw $32,    %bx
    jmp 1b
2:
    hlt
    jmp 2b
3:  # dirent found
    addw $26,    %bx    /* offset 26: DIR_FstClusLO.
		    	  (DIR_FstClusHi is always 0 for FAT12) */
    movw %fs:(%bx), %ax
    movw %ax,    CurrentClusterNum

    /* read FAT to %fs:0000 (root dir. is overwritten) */
    movw 0x0E,   %ax	# 0x0E: BPB_RsvdSecCnt
    call LBA2CHS
    movw $0,     %bx 	# destination is %es:%bx
    movb $0x00,  %dl    # drive number. bit 7 must be 0 for floppy. 
    movb $0x02,  %ah    # command:	read diskette sectors
    movb 0x16,   %al    # the number of sectors to read(0x16: BPB_FATSz16)
    int  $0x13	      	# BIOS floppy service

    /* set EndOfCluster (typically 0x0FFF) */
    movw %fs:1,  %ax
    andw $0x0FFF, %ax
    movw %ax,    EndOfCluster

    /* set Sec512PerClus (typically 1) */
    movb 0x0D,   %al    # 0x0D = FAT_BPB.SecPerClus
    movb $0,     %ah
    movw 0x0B,   %bx    # 0x0B= BPB_BytsPerSec 
    mulw %bx            # %ax * %bx => %dx:%ax (%dx is always 0 for FD)
    shrw $9,     %ax
    movw %ax,    Sec512PerClus

    /* set FirstDataSector.
	FirstDataSector = FirstRootDirSecNum + RootDirSectors */
    movw FirstRootDirSecNum, %ax
    addw RootDirSectors, %ax
    movw %ax,    FirstDataSector

    /* read all clusters for "KERNEL.BIN" */
    movw $0,     %cx
    movw CurrentClusterNum, %bx
1:
    # print dot
    movb $'.',   %al
    movb $0x0E,  %ah
    int  $0x10

    call read_cluster
    call get_FAT12_entry
    cmpw EndOfCluster, %bx
    je  2f

    incw %cx
    jmp 1b
2:	
#if 0
    popw %dx
    popw %cx
    popw %bx
    popw %ax
#endif
    ret

/* Calculate CHS(%ch, %dh, %cl) from LBA(%ax).
	tmp      = lba % SECTORS_PER_CYLINDER
	cylinder = lba / SECTORS_PER_CYLINDER
	head     = tmp / SECTORS_PER_TRACK
	sector   = tmp % SECTORS_PER_TRACK + 1 */
LBA2CHS:
    pushw %ax
    pushw %bx
    movb  $36,    %bl        # 36 is #sectors per cylinder for 3.5inch FD
    divb  %bl                # %al=quotient, %ah=remainder
    movb  %al,    %ch        # set track (cylinder) number
    movb  %ah,    %al        # %al = tmp
    movb  $0,     %ah
    movb  $18,    %bl        # 18 is #sectors per track for 3.5inch FD
    divb  %bl
    movb  %al,    %dh        # set head number
    movb  %ah,    %cl
    incb  %cl                # set sector number
    popw  %bx
    popw  %ax
    ret

/* input: cluster number(%bx) assuming FAT is located from %fs:0000,
   output: its next cluster number (%bx) */
get_FAT12_entry:
    pushw %ax
    movw  %bx,    %ax
    shrw  $1,     %bx
    addw  %ax,    %bx     /* FATOffset(%bx) = clus_num + (clus_num / 2) */
    movw  %fs:(%bx), %bx
    testw $1,     %ax
    jz    1f
    /* clus_num is odd (see p.17 in fatgen103.pdf) */
    shrw  $4,     %bx
    jmp   2f
1:  /* clus_num is even */
    andw  $0x0FFF, %bx
2:	
    popw  %ax
    ret

/* input: clus_num(%bx), count(%cx).
   read the cluster to (%gs+(count * SecPerClus * BPB_BytsPerSec)/16:0000) */
read_cluster:
    pushw %ax
    pushw %bx
    pushw %cx
    pushw %dx

    /*  Calculate the sector num for the cluster.
	FirstDataSector + (clus_num - 2) * FAT_BPB.SecPerClus */
    subw  $2,     %bx
    movw  %bx,    %dx
    movb  0x0D,   %al   # 0x0D = FAT_BPB.SecPerClus
    movb  $0,     %ah
    mulw  %dx           # %ax * %dx -> %dx:%ax (%dx is always 0 for FD)
    addw  FirstDataSector, %ax
    movw  %ax,    read_cluster_tmp

    /*  set destination %es */
    movw  Sec512PerClus, %ax
    shlw  $5,     %ax
    mulw  %cx           # %ax * %cx => %dx:%ax (%dx is always 0 for FD)
    movw  %gs,    %dx
    addw  %dx,    %ax   # %ax += (0x20 * %cx)
    movw  %ax,    %es

    movw  read_cluster_tmp, %ax
    call  LBA2CHS       # set %ch, %cl, %dh

    movw  $0,     %bx 	# destination is %es:%bx
    movb  $0x00,  %dl   # drive number. bit 7 must be 0 for floppy. 
    movb  $0x02,  %ah   # command:	read diskette sectors
    movb  0x0D,   %al	# 0x0D: BPB_SecPerClus (typically 1 for FD)
                        # %al: the number of sectors to read
    int   $0x13	      	# BIOS floppy service

    popw %dx
    popw %cx
    popw %bx
    popw %ax
    ret	

/* for lgdt gdt_selector */
    .align 2
gdt_selector:
    .word 0xFFFF       # GDT's limit
    .long gdt + 0x7c00 # GDT's 32-bit linear address

    .align 16
gdt:
    # for selector 0x00 (null selector)
    .word 0, 0, 0, 0

    # for selector 0x08
    .word 0xFFFF, 0x0000, 0x9A00, 0x00CF
    # limit=4GB ((0xFFFFF+1)*4KB=4GB), base=0x00000000
    # P=1, RPL=00, S=1
    # type=1010 (code, not-conforming, read/exec, not accessed)
    # granularity=4096, 386
    # G=1 (limit/4KB unit), D/B=1 (32bit), 0, AVL=0

    # for selector 0x10
    .word 0xFFFF, 0x0000, 0x9200, 0x00CF
    # limit=4GB ((0xFFFFF+1)*4KB=4GB), base=0x00000000
    # P=1, RPL=00, S=1
    # type=0010 (data, extend to high, read/write, not accessed)
    # G=1 (limit/4KB unit), D/B=1 (32bit), 0, AVL=0

/* for lidt idt_selector */
    .align 2
idt_selector:
    .word 0            # IDT's limit
    .long 0            # IDT's address, segment.

boot_file:
    .asciz "KERNEL  BIN"

.org 510               # place the following at offset of 510 bytes
    .word 0xAA55       # magic number for boot sector

/* the following variables are not loaded, but it does not matter
   since the initialized values (0) of them are not used. */
MemorySize:
    .word 0            # this must be here to be referred in main.c

EndOfCluster:
    .word 0

read_cluster_tmp:
    .word 0

Sec512PerClus:
    .word 0
	
CurrentClusterNum: 
    .word 0

FirstRootDirSecNum:	
    .word 0

RootDirSectors:
    .word 0

FirstDataSector:
    .word 0

/* end-of-file */
