/*
    udos: micro disk OS.
    Copyright (c) 2003, 2004   Katsuhiko Gondow (gondow@cs.titech.ac.jp)
    $Id: boot32.S,v 1.10 2004/07/14 05:43:19 gondow Exp gondow $
*/
#include "syscall.h"

.code32
    /* this code starts from 0x10000 */
.text
    .globl boot32_start
boot32_start:		
    # setup segment registers
    movw $0x10,   %ax
    movw %ax,     %ds
    movw %ax,     %es
    movw %ax,     %fs
    movw %ax,     %gs
    movw %ax,     %ss
    movl $0x9FF0, %esp   # use the first 64KB as kernel stack
    #movw $0x3FFFF0, %esp

    call _cmain	   # jump C program
boot32_end:	
    # never reached here
    hlt
    jmp boot32_end

/* all the below is not concerned with booting; misc assembly code */
/* ======================================================================== */
#define  SAVE_SEG_REGS     pushw %ds; pushw %es; pushw %fs; pushw %gs
#define  SET_SEG_REGS      movw  %ss, %ax; movw  %ax, %ds; movw  %ax, %es; \
			   movw  %ax, %fs; movw  %ax, %gs
#define  RESTORE_SEG_REGS  popw %gs; popw %fs; popw %es; popw %ds
    /* memo: pusha = push %eax, %ecx, %edx, %ebx, %esp, %ebp, %esi, %edi */
#define  HANDLER_PROLOGUE  SAVE_SEG_REGS; pusha; pushl 40(%esp); SET_SEG_REGS
#define  HANDLER_EPILOGUE  addl $4, %esp; popa; RESTORE_SEG_REGS; \
		           addl $4, %esp; iret

/* Note: to make stack modification easier, dummy error code might
	be pushed to unify the stack layouts. */

/* a wrapper function for `kbd_intr_handler' in vga.c. */
    .globl _kbd_intr_handler_entry
_kbd_intr_handler_entry:
    pushl $0xDEADBEEF	      # dummy error code
    HANDLER_PROLOGUE
    call _kbd_intr_handler    # defined in vga.c
    HANDLER_EPILOGUE

    .globl _pit_intr_handler_entry
_pit_intr_handler_entry:
    pushl $0xDEADBEEF	      # dummy error code
    HANDLER_PROLOGUE
    call _pit_intr_handler    # defined in vga.c
    HANDLER_EPILOGUE

    .globl _fdc_intr_handler_entry
_fdc_intr_handler_entry:
    pushl $0xDEADBEEF	      # dummy error code
    HANDLER_PROLOGUE
    call _FDC_intr_handler    # defined in fd.c
    HANDLER_EPILOGUE

    .globl _page_fault_handler_entry
_page_fault_handler_entry:
    HANDLER_PROLOGUE
    call _page_fault_handler  # defined in paging.c
    HANDLER_EPILOGUE

    .globl _tss_fault_handler_entry
_tss_fault_handler_entry:
    HANDLER_PROLOGUE
    call _tss_fault_handler   # defined in task.c
    HANDLER_EPILOGUE

    .globl _gp_fault_handler_entry
_gp_fault_handler_entry:
    HANDLER_PROLOGUE
    call _gp_fault_handler   # defined in task.c
    HANDLER_EPILOGUE

    .globl _system_call_entry
_system_call_entry:	
    pushl $0xDEADBEEF	      # dummy error code
    HANDLER_PROLOGUE
    call _system_call         # defined in task.c
    addl $36, %esp            # popup error code and general regs.
    RESTORE_SEG_REGS   
    addl $4,  %esp            # popup error code
    # here %eax has the return value from system_call
    iret

#  Note: %eax, %ecx, %edx are caller-save regs, not callee-save.
    .globl _ksetjmp
_ksetjmp:	    
    movl 4(%esp), %eax	    # &jmp_buf
    movl %ebx,    0(%eax)
    movl %esi,    4(%eax)
    movl %edi,    8(%eax)
    movl %esp,    12(%eax)
    movl %ebp,    16(%eax)
    movl 0(%esp), %edx
    movl %edx,    20(%eax)  # return address
    movl $0,      %eax      # return value
    ret

    .globl _klongjmp
_klongjmp:	    
    movl 4(%esp),  %edx	    # &jmp_buf
    movl 8(%esp),  %eax     # return value
    movl 0(%edx),  %ebx
    movl 4(%edx),  %esi
    movl 8(%edx),  %edi
    movl 12(%edx), %esp
    movl 16(%edx), %ebp
    movl 20(%edx), %ecx
    movl %ecx,     0(%esp)
    ret

    .global _test_entry
_test_entry:
    jmp _test_entry
    iret

.data
    .globl _kernel_page_dir
    .org 0x1000
_kernel_page_dir:    # used in paging.c
    .fill 4096, 1, 0 # 4KB

    .globl _kernel_page_tbl_IM
    .org 0x2000
_kernel_page_tbl_IM: # used in paging.c
    .fill 4096, 1, 0 # 4KB
