/*
    tool for udos: micro disk OS.
    Copyright (c) 2003, 2004   Katsuhiko Gondow (gondow@cs.titech.ac.jp)
    $Id: exe2bin.c,v 1.3 2004/09/21 05:26:22 gondow Exp gondow $
 */
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <bfd.h>

// #define DEBUG

/* ---------------------------------------------------------------------- */
static void *emalloc (size_t size);
static void write_section_contents (bfd *abfd, asection *sec, FILE *fp);
#if defined(DEBUG)
static void print_section_info (bfd *abfd, asection *sec);
#endif
/* ---------------------------------------------------------------------- */
static void *
emalloc (size_t size)
{
    void *p = malloc (size);
    if (p == NULL) perror ("malloc");
    return p;
}

#if defined(DEBUG)
static void
print_section_info (bfd *abfd, asection *sec)
{
    unsigned int opb = bfd_octets_per_byte (abfd);
    printf ("section = %5s, vma = %lx, lma = %lx, size = 0x%-5lx, "
	    "fileoff = %llx\n",
	    sec->name, sec->vma, sec->lma, bfd_section_size (abfd, sec)/opb,
	    sec->filepos);
}
#endif

static void
write_section_contents (bfd *abfd, asection *sec, FILE *fp)
{
    bfd_byte *buf = NULL;
    bfd_size_type size;
    long offset;
    bfd_boolean result;

    size = bfd_section_size (abfd, sec)/bfd_octets_per_byte (abfd);
    buf = emalloc (size);
    result = bfd_get_section_contents (abfd, sec, buf, 0, size);
    if (result == FALSE) {
	fprintf (stderr, "Fail: bfd_get_section_contents\n");
	exit (1);
    }

    offset = (long)sec->lma - (long)abfd->start_address;
#if defined(DEBUG)
    printf ("offset=%x, size=%x\n", (int)offset, (int)size);
#endif
    fseek (fp, offset, SEEK_SET);
    fwrite (buf, size, 1, fp);
    free (buf);
}
/* ---------------------------------------------------------------------- */
int
main (int argc, char *argv [])
{
    enum { MAX_SECTION = 5 };
    asection *section [MAX_SECTION];
    char *section_name [MAX_SECTION] = {".text", ".data", ".rdata", ".idata", ".bss"};
    bfd *abfd; int i; FILE *fp;

    if (argc < 3) {
	fprintf (stderr, "Usage: %s kernel image\n", argv [0]);
	fprintf (stderr, "This program builds kernel's memory image file.\n");
	exit (1);
    }

    fp = fopen (argv [2], "w");
    if (fp == NULL) {
	fprintf (stderr, "Cannot open file: %s\n", argv [2]);
	exit (1);
    }

    bfd_init ();
    abfd = bfd_openr (argv [1], "default");
    bfd_check_format (abfd, bfd_object);
#if defined(DEBUG)
    printf ("start address = %lx\n", abfd->start_address);
#endif
    for (i = 0; i < MAX_SECTION; i++) {
	section [i] = bfd_get_section_by_name (abfd, section_name [i]);
	if (section [i] == NULL)
	    continue;
#if defined(DEBUG)
	print_section_info (abfd, section [i]);
#endif
	write_section_contents (abfd, section [i], fp);
    }

    bfd_close (abfd);
    fclose (fp);
    return 0;
}
/* ---------------------------------------------------------------------- */
