/*
    udos: micro disk OS.
    Copyright (c) 2003, 2004   Katsuhiko Gondow (gondow@cs.titech.ac.jp)
    $Id: fs.c,v 1.9 2004/06/23 07:34:44 gondow Exp gondow $
*/
#include "misc.h"
#include "fs.h"
#include "fat12.h"
#include "vga.h"
#include "task.h"

/* ---------------------------------------------------------------------- */
enum {
    MAX_SYSTEM_FILE = 64,
};
/* ---------------------------------------------------------------------- */
struct file_ops file_ops [] = {
    {FAT12_open,   FAT12_close  , FAT12_read  , FAT12_write,   FAT12_seek},
    {console_open, console_close, console_read, console_write, NULL      }
};

struct file file [MAX_SYSTEM_FILE];
/* ---------------------------------------------------------------------- */
void
fs_init   (void)
{
    int i;
    for (i = 0; i < MAX_SYSTEM_FILE; i++) {
	file [i].ref_count = 0;
    }
}

int
sys_open   (const char *path, int flag)
{
    int fd, i, ret;
    struct process *process = &process_table [sys_getpid ()];

    for (fd = 0; fd < MAX_PROCESS_FILE; fd++) {
	if (process->file [fd] == NULL)
	    break;
    }

    if (fd >= MAX_PROCESS_FILE) {
	printk ("sys_open: no more file table");
	return -1;
    }

    for (i = 0; i < MAX_SYSTEM_FILE; i++) {
	if (file [i].ref_count == 0) {
	    process->file [fd] = &file [i];
	    break;
	}
    }

    if (i >= MAX_SYSTEM_FILE) {
	printk ("sys_open: no more struct file");
	return -1;
    }

    if (!strcmp (path, "/dev/console"))
	process->file [fd]->file_ops = &file_ops [1]; /* console */
    else
	process->file [fd]->file_ops = &file_ops [0]; /* FAT12 */

    process->file [fd]->ref_count++;
    ret = process->file [fd]->file_ops->open (process->file [fd], path, flag);

    if (ret == -1) {
	process->file [fd]->ref_count--;
	process->file [fd] = NULL;
	return -1;
    } else {
	return fd;
    }
}

int
sys_close  (int fd)
{
    struct process *process = &process_table [sys_getpid ()];
    int ret;
    if (fd < 0) {
	printk ("fd=%d\n", fd);
	panic ("sys_close illegal file descriptor");
    }
    if (process->file [fd] == NULL)
	return -1;
    ret = process->file [fd]->file_ops->close (process->file [fd]);
    process->file [fd]->ref_count--;
    if (process->file [fd]->ref_count == 0) {
	process->file [fd] = NULL;
    }
    return ret;
}

int
sys_read   (int fd, void *buf, int count)
{
    struct process *process = &process_table [sys_getpid ()];
    if (fd < 0 || process->file [fd] == NULL) {
	printk ("fd=%d\n", fd);
	panic ("sys_read illegal file descriptor");
    }
    return process->file [fd]->file_ops->read (process->file [fd], buf, count);
}

int
sys_write  (int fd, void *buf, int count)
{
    struct process *process = &process_table [sys_getpid ()];
    if (fd < 0 || process->file [fd] == NULL) {
	printk ("fd=%d\n", fd);
	panic ("sys_write illegal file descriptor");
    }
    return process->file [fd]->file_ops->write (process->file [fd], buf, count);
}

int
sys_seek   (int fd, int offset, int whence)
{
    struct process *process = &process_table [sys_getpid ()];
    if (fd < 0 || process->file [fd] == NULL) {
	printk ("fd=%d\n", fd);
	panic ("sys_seek illegal file descriptor");
    }
    return process->file [fd]->file_ops->seek (process->file [fd],
					       offset, whence);
}

int
sys_readdir (int fd, struct FAT_dirent *dirent)
{
    struct process *process = &process_table [sys_getpid ()];
    if (fd < 0 || process->file [fd] == NULL) {
	printk ("fd=%d\n", fd);
	panic ("sys_readdir illegal file descriptor");
    }
    return FAT12_readdir (process_table [sys_getpid ()].file [fd], dirent, 1);
}

int
sys_mkdir   (const char *path)
{
    return FAT12_mkdir (path);
}

int
sys_rmdir   (const char *path)
{
    return FAT12_rmdir (path);
}

int
sys_unlink  (const char *path)
{
    return FAT12_unlink (path);
}

int
sys_stat    (const char *path, struct FAT_dirent *dirent)
{
    return FAT12_stat (path, dirent);
}

void
sys_sync    (void)
{
    FAT12_sync ();
}
/* ---------------------------------------------------------------------- */
