/*
    udos: micro disk OS.
    Copyright (c) 2003, 2004   Katsuhiko Gondow (gondow@cs.titech.ac.jp)
    $Id: fs.h,v 1.5 2004/06/23 07:34:44 gondow Exp $
*/

#ifndef UDOS_FS_H
#define UDOS_FS_H
/* ---------------------------------------------------------------------- */
enum {
    O_RDONLY = 0x0,
    O_WRONLY = 0x1,
    O_RDWR   = 0x2,
    O_RWMASK = 0x3,
    O_APPEND = 0x10,
    O_CREAT  = 0x20,
    O_TRUNC  = 0x40,

#if defined(KERNEL) /* to avoid the conflict with <stdio.h> */
    SEEK_SET = 1,
    SEEK_CUR = 2,
    SEEK_END = 3,
#endif /* KERNEL */

    FILENAME_LEN   = 11,
};

struct FAT_dirent {
    char name [FILENAME_LEN + 1]; /* 8 + 3 + 1 (for '\0') */
    int  attr;
    int  clus_num;
    int  size;
    struct {
	int clus_num;
	int clus_offset;
    } entry;	/* this dirent is located in this place. */
};

struct file {
    int flag;
    int current_offset;
    int current_clus_num, current_clus_offset;
    int ref_count;
    struct file_ops *file_ops;
    struct FAT_dirent dirent;
};

struct file_ops {
    int (*open)  (struct file *fp, const char *path, int flag);
    int (*close) (struct file *fp);
    int (*read)  (struct file *fp, void *buf, int count);
    int (*write) (struct file *fp, void *buf, int count);
    int (*seek)  (struct file *fp, int offset, int whence);
};
/* ---------------------------------------------------------------------- */
void fs_init    (void);

int  sys_open    (const char *path, int flag);
int  sys_close   (int fd);
int  sys_read    (int fd, void *buf, int count);
int  sys_write   (int fd, void *buf, int count);
int  sys_seek    (int fd, int offset, int whence);

int  sys_readdir (int fd, struct FAT_dirent *dirent);
int  sys_mkdir   (const char *path);
int  sys_rmdir   (const char *path);
int  sys_unlink  (const char *path);
int  sys_stat    (const char *path, struct FAT_dirent *dirent);
void sys_sync    (void);
/* ---------------------------------------------------------------------- */
#endif /* UDOS_FS_H */
