/*
    udos: micro disk OS.
    Copyright (c) 2003, 2004   Katsuhiko Gondow (gondow@cs.titech.ac.jp)
    $Id: list.c,v 1.2 2004/06/23 07:34:44 gondow Exp $
*/
/* ================================================================== */
#include "misc.h"
#include "list.h"
#include "paging.h" /* for kmalloc */

/* CDLL: circular doubly-linked list.

   The first CDLL element is always used as anchor, i.e., 
   a pointer to the list, and logically not a member of the list.
       null list: CDLL
       singleton: CDLL->CDLL
       3 elems. : CDLL->CDLL->CDLL->CDLL
                 anchor head        tail
    anchor's data is always ignored (not used for any purpose).
    This structure makes this file's code simpler.
 */
/* ================================================================== */
static void CDLL_dump        (struct CDLL *head);
/* ================================================================== */
static void
CDLL_dump        (struct CDLL *head)
{
    struct CDLL *list;

    for (list = head->next; list != head; list = list -> next) {
	printk ("%d\n", *(int *)list->data);
    }
}

struct CDLL *
CDLL_create      (void *data)
{
    struct CDLL *cdll = kmalloc (sizeof (struct CDLL));
    cdll->data = data;
    cdll->next = cdll->prev = cdll;
    return cdll;
}

void
CDLL_wrap        (struct CDLL *cdll, void *data)
{
    cdll->data = data;
    cdll->next = cdll->prev = cdll;
}

void
CDLL_insert_head (struct CDLL *head, struct CDLL *cdll)
{
    cdll->prev = head;
    cdll->next = head->next;
    head->next->prev = cdll;
    head->next = cdll;
}

void
CDLL_insert_tail (struct CDLL *head, struct CDLL *cdll)
{
    cdll->prev = head->prev;
    cdll->next = head;
    head->prev->next = cdll;
    head->prev = cdll;
}

void
CDLL_insert_sorted (struct CDLL *head, struct CDLL *cdll,
		    int (*compare)(void *, void*))
{
    struct CDLL *list;
    if (CDLL_is_null (head)) {
	CDLL_insert_head (head, cdll);
	return;
    }
    for (list = head->next; list != head; list = list -> next) {
	if (compare (list->data, cdll->data)) {
	    CDLL_insert_head (list->prev, cdll);
	    return;
	}
    }
    CDLL_insert_tail (head, cdll);
}

void
CDLL_remove      (struct CDLL *cdll)
{
    cdll->next->prev = cdll->prev;
    cdll->prev->next = cdll->next;
    cdll->next = cdll->prev = NULL;
}

int
CDLL_is_null     (struct CDLL *head)
{
    return head == head->next;
}

int
CDLL_is_single   (struct CDLL *head)
{
    return !CDLL_is_null (head) && (head->next->next == head);
}
/* ================================================================== */
