/*
    udos: micro disk OS.
    Copyright (c) 2003, 2004   Katsuhiko Gondow (gondow@cs.titech.ac.jp)
    $Id: main.c,v 1.21 2004/06/23 07:34:44 gondow Exp gondow $
*/
#include "vga.h"
#include "cpu.h"
#include "paging.h"
#include "misc.h"
#include "task.h"
#include "fd.h"
#include "fat12.h"

/* ---------------------------------------------------------------------- */
int cmain (void);
/* ---------------------------------------------------------------------- */
int
cmain (void)
{
    kbd_enable_A20gate ();
    setup_PIC8259A ();
    setup_PIT8254 (PIT_COUNT);
    setup_KBD8042 ();
    // setup_RTC146818A ();
    check_memory_size ();    /* set memory_size */
    setup_GDT_again ();
    setup_IDT_again ();
    vga_screen_clear ();
    setup_paging ();
    setup_kmalloc ();
    task_init ();

    /* move the kernel stack to KERNEL_STACK_ADDR */
    kmem_map (KERNEL_STACK_ADDR);
    kmem_map (KERNEL_STACK_ADDR - 4);
    asm volatile ("movl %0, %%esp"::"r"(KERNEL_STACK_ADDR));

    /* enable intr: (IRQ0(timer), IRQ1(kbd), IRQ6(fdc)) */
    outb (inb (0x21) & ~((1<<0)+(1<<1)+(1<<6)), 0x21);
    asm volatile ("sti");

    if (sys_fork () == 0) {
	/* another process is required for the kernel to block */
	FDC_init ();
	buffer_init ();
	FAT12_init ();
	fs_init ();
	printk ("Welcome to udos 0.0.1 (%dMB)\n", memory_size);
	sys_exec ("/bin/init.bin", NULL);
    }

    /* kernel process MUST NOT call exit(); run forever as idle process */
    while (1)
	yield (PROCESS_READY);
    panic ("never reached here");
    return 0; /* to inhibit compiler caution */
}
/* ---------------------------------------------------------------------- */
