/*
    udos: micro disk OS.
    Copyright (c) 2003, 2004   Katsuhiko Gondow (gondow@cs.titech.ac.jp)
    $Id: misc.c,v 1.9 2004/06/23 07:59:03 gondow Exp $
*/
#include "misc.h"
#include "vga.h"
#include "paging.h"
#include "task.h"
/* ---------------------------------------------------------------------- */
void
printk  (const char *fmt, ...)
{
    uint32_t *argp = (void *)&fmt;
    while (1) {
	switch (*fmt) {
	case '\0':
	    return;
	case '%':
	    fmt++; argp++;
	    switch (*fmt) {
	    case '%': vga_put_char         ('%'); argp--;       break;
	    case 'd': vga_put_signed_num   (*argp, 10);         break;
	    case 'x': /* falling through */
	    case 'p': vga_put_unsigned_num (*argp, 16);         break;
	    case 'c': vga_put_char         (*argp);             break;
	    case 's': vga_put_string       ((char *)*argp);     break;
	    default:  panic ("printk");                         break;
	    }
	    break;
	default:
	    vga_put_char (*fmt);
	    break;
	}
	fmt++;
    }
}

void
fwrite (const char *buf, int size)
{
    while (size--)
	vga_put_char (*buf++);
}

void
_panic  (char *mesg, char *file, int line)
{
    printk ("PANIC@%s,%d:%s\n", file, line, mesg);
    dump_stack ();
    asm volatile ("cli; 1: hlt; jmp 1b");
}

void *
memcpy (void *dest, const void *src, uint32_t size)
{
    int i;
    for (i = 0; i < size; i++) {
	*(uint8_t *)dest++ = *(uint8_t *)src++;
   }
    return dest;
}

void *
memset (void *dest, int ch, uint32_t size)
{
    int i;
    for (i = 0; i < size; i++) {
	*(uint8_t *)dest++ = ch;
    }
    return dest;
}

int
strcmp (const char *s1, const char *s2)
{
    while (1) {
	if (*s1 != *s2 || *s1 == '\0' || *s2 == '\0')
	    return *s1 - *s2;
	s1++, s2++;
    }
    panic ("strcmp");
}

int
strncmp (const char *s1, const char *s2, unsigned int len)
{
    while (1) {
	if (len-- == 0)
	    return 0;
	if (*s1 != *s2 || *s1 == '\0' || *s2 == '\0')
	    return *s1 - *s2;
	s1++, s2++;
    }
    panic ("strncmp");
}

int
strlen (const char *s)
{
    int i = 0;
    while (*s != '\0') {
	s++;
	i++;
    }
    return i;
}

char *
strchr (const char *s, int c)
{
    while (*s != '\0') {
	if (*s == c)
	    return (char *)s;
	s++;
    }
    return NULL;
}

char *
strrchr (const char *s, int c)
{
    const char *p = NULL;
    while (*s != '\0') {
	if (*s == c)
	    p = s;
	s++;
    }
    return (char *)p;
}

char *
strdup (const char *s)
{
    char *p = kmalloc (strlen (s) + 1);
    memcpy (p, s, strlen (s) + 1);
    return p;
}

char *
strcpy (char *s1, const char *s2)
{
    memcpy (s1, s2, strlen (s2) + 1);
    return s1;
}
/* ---------------------------------------------------------------------- */
