/*
    udos: micro disk OS.
    Copyright (c) 2003, 2004   Katsuhiko Gondow (gondow@cs.titech.ac.jp)
    $Id: task.h,v 1.26 2004/06/23 07:34:44 gondow Exp $
*/
#ifndef UDOS_TASK_H
#define UDOS_TASK_H
/* ---------------------------------------------------------------------- */
#include "cpu.h"
#include "misc.h"
#include "list.h"
#include "fs.h"
#include "boot32.h"
/* ---------------------------------------------------------------------- */
enum {
    KERNEL_STACK_ADDR = 0x7FFFF000,
    USER_CODE_ADDR    = 0x80000000,
    USER_STACK_ADDR   = 0xFFFFF000,

    MAX_PROCESS      = 10,
    MAX_PROCESS_FILE = 32,
    MAX_SIGNAL       = 4,

    UNINTERRUPTIBLE  = 0,
    INTERRUPTIBLE    = 1,
};

enum Process_Status {
    PROCESS_UNUSED,
    PROCESS_RUNNING,
    PROCESS_READY,
    PROCESS_ZOMBIE,
    PROCESS_BLOCKED,
};

struct signal {
    int is_received;
    void (*handler)(int); 
};

struct process {
    enum Process_Status status;
    int          pid, ppid;
    struct TSS  *tss;
    int          exit_status;
    struct CDLL  cdll_children, *cdll_self;

    uint64_t     wakeup_time;   /* used by msleep */
    void        *sync_ident;    /* used by sync_wait/sync_notify */
    int          is_interruptible;

    struct file *file [MAX_PROCESS_FILE];
    int          current_dir;
    struct signal signal  [MAX_SIGNAL];
};

enum {
    SIGSEGV = 0,
    SIGINT  = 1,
    SIGUSR1 = 2,
    SIGUSR2 = 3,
};
/* ---------------------------------------------------------------------- */
extern struct process process_table [];
extern jmp_buf syscall_jmp_buf;
/* ---------------------------------------------------------------------- */
void dump_process_table (void);
void dump_stack  (void);
void tss_fault_handler (uint32_t error);
void gp_fault_handler  (uint32_t error);
void task_init   (void);
void yield       (int status);
int  check_signals (int pid);
void process_signals (int do_klongjmp);
void check_sleeping_processes (void);
int  sync_wait   (void *ident, int timeout_ms, int is_interruptible);
void sync_notify (void *ident);
void setup_signal_handler (void *handler, int signal);
int  sys_exec    (char *path, char *argv []);
int  sys_fork    (void);
void sys_exit    (int status);
int  sys_wait    (int *status);
int  sys_msleep  (int ms);
int  sys_getpid  (void);
int  sys_getppid (void);
int  sys_chdir   (const char *path);
void (*sys_signal (int sig_num, void (*handler)(int),
		  void (*caller)(void)))(int);
int  sys_kill    (int pid, int sig_num);
int  sys_tcgetpid (int fd);
int  sys_tcsetpid (int fd, int pid);
int  system_call (int dummy1, int arg4, int arg5, int dummy2, int dummy3,
		  int arg1, int arg3, int arg2, int sym_num);
/* ---------------------------------------------------------------------- */
#endif /* UDOS_TASK_H */
