/*
    udos: micro disk OS.
    Copyright (c) 2003, 2004   Katsuhiko Gondow (gondow@cs.titech.ac.jp)
    $Id: vga.h,v 1.7 2004/06/23 07:34:44 gondow Exp $
*/

#ifndef UDOS_VGA_H
#define UDOS_VGA_H
#include "misc.h"
#include "fs.h"

/* PIT interval timer */
enum {
    PIT_COUNT = 0xFFFF,
    PIT_HZ    = (1193182 / PIT_COUNT),
    PIT_MS    = (1000 / PIT_HZ),    /* approx. millisecond of each interval */
};
extern uint64_t PIT_ticks;
extern uint16_t memory_size;
extern volatile int yield_is_required;

void     outb  (uint8_t value, uint16_t port);
uint8_t  inb   (uint16_t port);
void     outw  (uint16_t value, uint16_t port);
uint16_t inw   (uint16_t port);
void     delay (void);

void setup_PIC8259A    (void);
void setup_PIT8254     (uint16_t count);
void setup_KBD8042     (void);
void setup_RTC146818A  (void);
void check_memory_size (void);

void kbd_enable_A20gate (void);
void kbd_intr_handler  (void);
void pit_intr_handler  (void);

void vga_screen_clear  (void);
void vga_screen_scroll (void);
void vga_cursor_move   (void);
void vga_cursor_toggle (int on);
void vga_put_char      (int c);
void vga_put_string    (char *s);
void vga_put_unsigned_num (unsigned int v, int radix);
void vga_put_signed_num   (int v, int radix);

int  kbd_get_char      (void);
void kbd_get_line      (char *s);

int  console_open      (struct file *fp, const char *path, int flag);
int  console_close     (struct file *fp);
int  console_read      (struct file *fp, void *buf, int count);
int  console_write     (struct file *fp, void *buf, int count);

#endif /* UDOS_VGA_H */

