/*
    udos: micro disk OS.
    Copyright (c) 2003, 2004   Katsuhiko Gondow (gondow@cs.titech.ac.jp)
    $Id: libudos.c,v 1.7 2004/06/23 07:52:51 gondow Exp gondow $
*/
#include "udos.h"
#include "libudos.h"
/* ---------------------------------------------------------------------- */
static void put_unsigned_num  (int fd, unsigned int v, int radix);
/* ---------------------------------------------------------------------- */
static void
put_signed_num    (int fd, int v, int radix)
{
    if (v < 0) {
	write (fd, "-", 1);
	v *= -1;
    }
    put_unsigned_num (fd, v, radix);
}

static void
put_unsigned_num    (int fd, unsigned int v, int radix)
{
    enum {MAX = 16};
    int  i, h;
    char digit [MAX];

    if (v == 0) {
	write (fd, "0", 1);
	return;
    }
    for (i = 0; v != 0; i++) {
	if (i >= MAX)
	    break;
	h = v % radix;
	v = v / radix;
	digit [i] = ((h >= 10) ? ('A' + h - 10) : ('0' + h));
    }
    for (i--; i >= 0; i--) {
	write (fd, &digit [i], 1);
    }
}
/* ---------------------------------------------------------------------- */
void
init_console (void)
{
    int fd;
    fd = open ("/dev/console", O_RDONLY);
    assert (fd == 0);
    fd = open ("/dev/console", O_WRONLY);
    assert (fd == 1);
    fd = open ("/dev/console", O_WRONLY);
    assert (fd == 2);
}

void
fdgets (int fd, char *s, int size)
{
    int i;
    char c;
    for (i = 0; i < size; i++) {
	read (fd, &c, 1);
	if (c == '\n') break;
	*s++ = c;
    }
    *s = '\0';
}

void *
memcpy (void *dest, const void *src, uint32_t size)
{
    int i;
    for (i = 0; i < size; i++) {
	*(uint8_t *)dest++ = *(uint8_t *)src++;
   }
    return dest;
}

void *
memset (void *dest, int ch, uint32_t size)
{
    int i;
    for (i = 0; i < size; i++) {
	*(uint8_t *)dest++ = ch;
    }
    return dest;
}

int
atoi  (const char *s)
{
    int i, num = 0;
    for (i = 0; s [i] != '\0'; i++) {
	num = num * 10 + (s [i] - '0');
    }
    return num;
}

int
strlen (const char *s)
{
    int i = 0;
    while (*s != '\0') {
	s++;
	i++;
    }
    return i;
}

char *
strchr (const char *s, int c)
{
    while (*s != '\0') {
	if (*s == c)
	    return (char *)s;
	s++;
    }
    return NULL;
}

char *
strrchr (const char *s, int c)
{
    const char *p = NULL;
    while (*s != '\0') {
	if (*s == c)
	    p = s;
	s++;
    }
    return (char *)p;
}

int
strcmp (const char *s1, const char *s2)
{
    while (1) {
	if (*s1 != *s2 || *s1 == '\0' || *s2 == '\0')
	    return *s1 - *s2;
	s1++, s2++;
    }
    panic ("strcmp");
}

int
strncmp (const char *s1, const char *s2, unsigned int len)
{
    while (1) {
	if (len-- == 0)
	    return 0;
	if (*s1 != *s2 || *s1 == '\0' || *s2 == '\0')
	    return *s1 - *s2;
	s1++, s2++;
    }
    panic ("strncmp");
}

char *
strcpy (char *s1, const char *s2)
{
    memcpy (s1, s2, strlen (s2) + 1);
    return s1;
}

/* unbuffered, restricted conversion specifiers, incorrect return-value */
int
printf  (const char *fmt, ...)
{
    uint32_t *argp = (void *)&fmt;
    char *s;
    while (1) {
	switch (*fmt) {
	case '\0':
	    return 0;
	case '%':
	    fmt++; argp++;
	    switch (*fmt) {
	    case '%': write (1, "%", 1); argp--;       break;
	    case 'd': put_signed_num   (1, *argp, 10); break;
	    case 'x': /* falling through */
	    case 'p': put_unsigned_num (1, *argp, 16); break;
	    case 'c': write (1, (char *)argp, 1);      break;
	    case 's':
		s = (char *)*argp;
		while (*s != 0)
		    write (1, s++, 1);
		break;
	    default:  
		printk ("printf: Unsupported conversion: %c\n", *fmt);
		return -1;
	    }
	    break;
	default:
	    write (1, fmt, 1);
	    break;
	}
	fmt++;
    }
    return 0;
}
/* ---------------------------------------------------------------------- */
