#include "udos.h"
#include "libudos.h"

/* ---------------------------------------------------------------------- */
enum {
    MAX_ARGS = 10,
};
/* ---------------------------------------------------------------------- */
static int str2argv (char *s, char *argv[], int argvlen);
static int do_internal_command (int argc, char *argv []);
static int search_executable (char *argv0, char *path);
static void handler (int sig);
/* ---------------------------------------------------------------------- */
static char *exec_path [] = {"/bin/", "./", NULL};
/* ---------------------------------------------------------------------- */
static int
str2argv (char *s, char *argv[], int argvlen)
{
    int i;
    char *t;

    for (i = 0; i < argvlen - 1; i++) {
	while (*s == ' ')
	    s++;
	if (*s == '\0') {
	    argv [i] = NULL;
	    break;
	}
	argv [i] = s;
	t = strchr (s, ' ');
	if (t != NULL) {
	    *t = '\0';
	    s = t + 1;
	} else {
	    argv [++i] = NULL;
	    break;
	}
    }
    return i;
}

static int
do_internal_command (int argc, char *argv [])
{
    int ret = 1;
    if (!strcmp (argv [0], "cd")) {
	if (argc < 2) chdir ("/"); else chdir (argv [1]);
    } else if (!strcmp (argv [0], "exit")) {
	exit (0);
    } else if (!strcmp (argv [0], "ps")) {
	dump_process_table ();
    } else if (!strcmp (argv [0], "mem")) {
	dump_mem_usage ();
    } else {
	ret = 0;
    }
    return ret;
}

static int
search_executable (char *argv0, char *path)
{
    int i, fd;
    for (i = 0; exec_path [i] != NULL; i++) {
	strcpy (path, exec_path [i]);
	strcpy (path + strlen (exec_path [i]), argv0);
	fd = open (path, O_RDONLY);
	if (fd >= 0) {
	    close (fd);
	    return 0;
	}
    }

    if (strchr (argv0, '.') != NULL)
	return -1;

    for (i = 0; exec_path [i] != NULL; i++) {
	strcpy (path, exec_path [i]);
	strcpy (path + strlen (exec_path [i]), argv0);
	strcpy (path + strlen (exec_path [i]) + strlen (argv0), ".bin");
	fd = open (path, O_RDONLY);
	if (fd != -1) {
	    close (fd);
	    return 0;
	}
    }
    return -1;
}
static
void handler (int sig)
{
    printf ("Segmentation fault (Zugan!)\n");
    exit (1);
}
/* ---------------------------------------------------------------------- */
int
cmain (void)
{
    char buf [512], path [512];
    char *argv [MAX_ARGS];
    int pid, argc, ret, i;

    signal (SIGSEGV, handler);
    for (i = 0; ; i++) {
	printf ("[%d]%% ", i);
	fdgets (0, buf, sizeof (buf));
	argc = str2argv (buf, argv, MAX_ARGS);
	if (argc == 0)
	    continue;
	if (do_internal_command (argc, argv))
	    continue;
	pid = fork ();
	if (pid == 0) {
	    tcsetpid (0, getpid ());
	    if (argv[0][0] == '/') {
		exec (argv [0], argv);
	    } else {
		ret = search_executable (argv [0], path);
		if (ret == 0)
		    exec (path, argv);
	    }
	    printf ("shell: No such file: |%s|\n", argv [0]);
	    exit (1);
	}
	wait (NULL);
    }
    assert (0);
    return 0;
}
/* ---------------------------------------------------------------------- */
