/*
    udos: micro disk OS.
    Copyright (c) 2003, 2004   Katsuhiko Gondow (gondow@cs.titech.ac.jp)
    $Id: udos.h,v 1.11 2004/06/23 07:52:51 gondow Exp $
*/
/* ---------------------------------------------------------------------- */
#ifndef UDOS_UDOS_H
#define UDOS_UDOS_H

#define panic(mesg)        _panic (mesg, __FILE__, __LINE__)
#define assert(val)        ((val) ? (void) 0 : panic ("assert failure"))

#define SIG_DFL  ((void (*)(int))0)
#define SIG_IGN  ((void (*)(int))1)

enum {
    O_RDONLY = 0x0,
    O_WRONLY = 0x1,
    O_RDWR   = 0x2,
    O_RWMASK = 0x3,
    O_APPEND = 0x10,
    O_CREAT  = 0x20,
    O_TRUNC  = 0x40,

    SEEK_SET = 1,
    SEEK_CUR = 2,
    SEEK_END = 3,

    FILENAME_LEN   = 11,
};

enum {
    ROOT_DIR_CLUS_NUM = 0,

    ATTR_READ_ONLY = 0x01,
    ATTR_HIDDEN    = 0x02,
    ATTR_SYSTEM    = 0x04,
    ATTR_VOLUME_ID = 0x08,
    ATTR_DIRECTORY = 0x10,
    ATTR_ARCHIVE   = 0x20,
    ATTR_LONG_NAME = ATTR_READ_ONLY | ATTR_HIDDEN 
                     | ATTR_SYSTEM | ATTR_VOLUME_ID,
    ATTR_LONG_NAME_MASK = ATTR_LONG_NAME | ATTR_DIRECTORY | ATTR_ARCHIVE,
};

enum {
    SIGSEGV = 0,
    SIGINT  = 1,
    SIGUSR1 = 2,
    SIGUSR2 = 3,
};
/* ---------------------------------------------------------------------- */
struct FAT_dirent {
    char name [FILENAME_LEN + 1]; /* 8 + 3 + 1 (for '\0') */
    int  attr;
    int  clus_num;
    int  size;
    struct {
	int clus_num;
	int clus_offset;
    } entry;	/* this dirent is located in this place. */
};
/* ---------------------------------------------------------------------- */
int  fork   (void);
int  exec   (char *path, char *argv []);
void exit   (int status);
int  wait   (int *status);
int  msleep (int ms);

int  open    (const char *path, int flag);
int  close   (int fd);
int  read    (int fd, void *buf, int count);
int  write   (int fd, void *buf, int count);
int  seek    (int fd, int offset, int whence);

int  readdir (int fd, struct FAT_dirent *dirent);
int  mkdir   (const char *path);
int  rmdir   (const char *path);
int  unlink  (const char *path);
void sync    (void);
int  chdir   (const char *path);
int  stat    (const char *path, struct FAT_dirent *dirent);
void (*signal (int sig, void (*handler)(int)))(int);
int  kill    (int pid, int sig_num);
int  getpid  (void);
int  getppid (void);
int  tcsetpid (int fd, int pid);
int  tcgetpid (int fd);

int  printk (const char *fmt, ...);
int  _panic (char *mesg, char *file, int line);
void yield   (int status);
void dump_mem_usage (void);
void dump_process_table (void);
void infinite_loop (void);
int  sync_wait  (void *ident, int timeout_ms);
void sync_notify (void *ident);
void dump_stack (void);
/* ---------------------------------------------------------------------- */
#endif /* UDOS_UDOS_H */
